"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var typescript_1 = __importDefault(require("typescript"));
var const_1 = require("./const");
var type_1 = require("./type");
var formProperFilePath = function (_a) {
    var filePath = _a.filePath;
    return filePath.split(const_1.separator).filter(Boolean).join(const_1.separator);
};
var checkJavaScriptFileExistByAppend = function (_a) {
    var filePath = _a.filePath;
    var _b = (0, const_1.extensionsUtil)().extensions, js = _b.js, mjs = _b.mjs;
    if (fs_1.default.existsSync("".concat(filePath).concat(js))) {
        return js;
    }
    if (fs_1.default.existsSync("".concat(filePath).concat(mjs))) {
        return mjs;
    }
    return false;
};
var isDirectory = function (filePath) {
    try {
        return fs_1.default.lstatSync(filePath).isDirectory();
    }
    catch (error) {
        return false;
    }
};
var addJSExtension = function (_a) {
    var filePath = _a.filePath, importPath = _a.importPath;
    if ((0, const_1.extensionsUtil)().matchAnyJs(filePath)) {
        return {
            procedure: 'skip',
        };
    }
    if (!isDirectory(filePath)) {
        var extension_1 = checkJavaScriptFileExistByAppend({
            filePath: filePath,
        });
        if (!extension_1) {
            return {
                procedure: 'skip',
            };
        }
        return {
            procedure: 'proceed',
            importPath: formProperFilePath({
                filePath: "".concat(importPath).concat(extension_1),
            }),
            filePathImported: formProperFilePath({
                filePath: "".concat(filePath).concat(extension_1),
            }),
        };
    }
    var extension = checkJavaScriptFileExistByAppend({
        filePath: path_1.default.posix.join(filePath, 'index'),
    });
    if (!extension) {
        return {
            procedure: 'skip',
        };
    }
    var file = "index".concat(extension);
    return {
        procedure: 'proceed',
        importPath: formProperFilePath({
            filePath: [importPath, const_1.separator, file].join(''),
        }),
        filePathImported: formProperFilePath({
            filePath: [filePath, const_1.separator, file].join(''),
        }),
    };
};
var traverseAndUpdateFileWithJSExtension = function (files) { return function (sourceFile) {
    var charactersDelimiter = '';
    var code = sourceFile.getText();
    var characters = code.split(charactersDelimiter);
    var replaceNodes = sourceFile.statements.flatMap(function (statement) {
        switch (statement.kind) {
            case typescript_1.default.SyntaxKind.ImportDeclaration:
            case typescript_1.default.SyntaxKind.ExportDeclaration: {
                var imExDeclaration_1 = statement;
                if (!imExDeclaration_1.moduleSpecifier) {
                    return [];
                }
                var moduleSpecifier = (0, type_1.asString)({
                    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                    // @ts-ignore
                    value: imExDeclaration_1.moduleSpecifier.text,
                    error: new Error('Module specifier of node should have text'),
                });
                var separator_1 = '/';
                var fileName = formProperFilePath({
                    filePath: !moduleSpecifier.endsWith(separator_1)
                        ? moduleSpecifier
                        : moduleSpecifier.slice(0, -1),
                })
                    .split(separator_1)
                    .pop();
                if (!fileName) {
                    throw new Error("Impossible for file name to be non-existent for ".concat(moduleSpecifier));
                }
                if (moduleSpecifier.startsWith('.')) {
                    var result = addJSExtension({
                        importPath: moduleSpecifier,
                        filePath: path_1.default.posix.join(sourceFile.fileName, '..', moduleSpecifier),
                    });
                    switch (result.procedure) {
                        case 'proceed': {
                            // if file name not included in list of js file read
                            var filePathImported_1 = result.filePathImported, importPath = result.importPath;
                            if (files.find(function (file) {
                                return file.endsWith(filePathImported_1);
                            })) {
                                var before = characters
                                    .filter(function (_, index) {
                                    return index > imExDeclaration_1.pos &&
                                        index < imExDeclaration_1.end;
                                })
                                    .join(charactersDelimiter);
                                return [
                                    {
                                        before: before,
                                        after: before.replace(moduleSpecifier, importPath),
                                    },
                                ];
                            }
                        }
                    }
                }
            }
        }
        return [];
    });
    return !replaceNodes.length
        ? []
        : [
            {
                file: sourceFile.fileName,
                code: replaceNodes.reduce(function (prev, _a) {
                    var before = _a.before, after = _a.after;
                    return prev.replace(before, after);
                }, code),
            },
        ];
}; };
exports.default = traverseAndUpdateFileWithJSExtension;
//# sourceMappingURL=traverse-and-update.js.map