import fs from 'fs';
import path from 'path';
import ts from 'typescript';
import traverseAndUpdateFileWithJSExtension from './traverse-and-update';
import { extensionsUtil, separator } from './const';
import Log from './log';
const getAllJSAndDTSFiles = (dir) => fs.readdirSync(dir).flatMap((file) => {
    const filePath = path.posix.join(dir.split(path.sep).join(separator), file);
    if (fs.statSync(filePath).isDirectory()) {
        return getAllJSAndDTSFiles(filePath);
    }
    return !extensionsUtil().matchAny(filePath) ? [] : [filePath];
});
const readCode = (files) => new Promise((resolve, reject) => {
    let fetchData = '';
    fs.createReadStream(files)
        .on('data', (data) => (fetchData = data.toString()))
        .on('end', () => resolve(fetchData))
        .on('error', reject);
});
const getAllJSAndDTSCodes = (files) => files.map(async (file) => ts.createSourceFile(file, await readCode(file), ts.ScriptTarget.ESNext));
export default class File {
    constructor() { }
    static create = () => new this();
    findMany = async ({ dir, include, }) => {
        // user may import files from `common` into `src`
        const files = include.concat(dir).flatMap(getAllJSAndDTSFiles);
        return (await Promise.all(getAllJSAndDTSCodes(files))).flatMap(traverseAndUpdateFileWithJSExtension(files));
    };
    writeMany = async ({ showChanges, withJSExtension, }) => {
        const repeat = withJSExtension.reduce((longestFileName, { file }) => longestFileName?.length <= file.length ? file : longestFileName, '').length;
        const log = !(withJSExtension.length && showChanges)
            ? undefined
            : Log.fromNumberOfFiles(withJSExtension.length);
        try {
            const errors = (await Promise.all(withJSExtension.map(({ code, file }) => new Promise((resolve) => fs.writeFile(file, code, (error) => {
                log?.increment({
                    repeat,
                    file,
                    succeed: !error,
                });
                resolve(!error
                    ? undefined
                    : {
                        file,
                        error,
                    });
            }))))).flatMap((element) => (!element ? [] : [element]));
            log?.end({ errors });
            return {
                type: 'done',
            };
        }
        catch (error) {
            return {
                type: 'error',
                error,
            };
        }
    };
}
//# sourceMappingURL=read-write.js.map